//
//  AppDelegate.swift
//

import UIKit
import UserNotifications
import Firebase
import AdLocus

@UIApplicationMain
class AppDelegate: UIResponder, UIApplicationDelegate, UNUserNotificationCenterDelegate, MessagingDelegate {

    var window: UIWindow?
    var pushService: ALPushService?

    var kGCMMessageIDKey: String { return "gcm.message_id" }

    func application(_ application: UIApplication, didFinishLaunchingWithOptions launchOptions: [UIApplication.LaunchOptionsKey: Any]?) -> Bool {

        // Firebase Cloud Message
        FirebaseApp.configure()
        Messaging.messaging().delegate = self

        // UserNotifications
        UNUserNotificationCenter.current().delegate = self
        let authOptions: UNAuthorizationOptions = [.alert, .badge, .sound]
        UNUserNotificationCenter.current().requestAuthorization(options: authOptions, completionHandler: { _, _ in })
        application.registerForRemoteNotifications()

        // Background Fetch
        if (UIDevice.current.systemVersion as NSString).floatValue >= 7.0 {
            UIApplication.shared.setMinimumBackgroundFetchInterval(1 * 60)
        }

        // AdLocus Push Service
        if pushService == nil {
            pushService = ALPushService.sharedInstance() as? ALPushService
            pushService?.key = "<AdLocus app key>"
            pushService?.fcmapIkey = "<FCM API KEY>"
            let targeting: ALTargeting = ALTargeting()
            pushService?.targeting = targeting
            pushService?.enable = true
        }

        return true
    }

    // MARK: - UISceneSession Lifecycle -
    @available(iOS 13.0, *)
    func application(_ application: UIApplication, configurationForConnecting connectingSceneSession: UISceneSession, options: UIScene.ConnectionOptions) -> UISceneConfiguration {
        return UISceneConfiguration(name: "Default Configuration", sessionRole: connectingSceneSession.role)
    }

    @available(iOS 13.0, *)
    func application(_ application: UIApplication, didDiscardSceneSessions sceneSessions: Set<UISceneSession>) {

    }

    // MARK: - Token & FCMkey -
    func application(_ application: UIApplication, didRegisterForRemoteNotificationsWithDeviceToken deviceToken: Data) {
        Messaging.messaging().apnsToken = deviceToken
    }

    func messaging(_ messaging: Messaging, didReceiveRegistrationToken fcmToken: String?) {
        print("Firebase registration token: \(fcmToken ?? "")")
        pushService?.fcMkey = fcmToken
    }

    // MARK: - AdLocus Push Service Handle -
    func application(_ application: UIApplication, performFetchWithCompletionHandler completionHandler: @escaping (UIBackgroundFetchResult) -> Void) {
        ALPushService.performFetch(completionHandler: completionHandler)
        completionHandler(.newData)
    }

    // Used by LBS Function
    func application(_ application: UIApplication, didReceiveRemoteNotification userInfo: [AnyHashable : Any], fetchCompletionHandler completionHandler: @escaping (UIBackgroundFetchResult) -> Void) {
        if let messageId = userInfo[kGCMMessageIDKey] {
            print("Message ID: \(messageId)")
        }
        ALPushService.performFetch(completionHandler: completionHandler, withUserinfo: userInfo)
    }

    func userNotificationCenter(_ center: UNUserNotificationCenter, willPresent notification: UNNotification, withCompletionHandler completionHandler: @escaping (UNNotificationPresentationOptions) -> Void) {
        if let userInfo = notification.request.content.userInfo as? [String : Any] {
            Messaging.messaging().appDidReceiveMessage(userInfo)
        }
        completionHandler([.badge, .sound, .alert])
    }

    // Click Notification Event
    func userNotificationCenter(_ center: UNUserNotificationCenter, didReceive response: UNNotificationResponse, withCompletionHandler completionHandler: @escaping () -> Void) {
        if let userInfo = response.notification.request.content.userInfo as? [String : Any], response.notification.request.trigger is UNPushNotificationTrigger {
            if let url = URL(string: userInfo["ad_link"] as? String ?? "") {
                DispatchQueue.main.async {
                    UIApplication.shared.open(url)
                }
            }
            completionHandler()
        } else {
            ALPushService.didReceiveLocalNotification(response.notification)
            completionHandler()
        }
    }

}
