//
//  NotificationService.swift
//

import UserNotifications
import AdLocus

class NotificationService: UNNotificationServiceExtension {

    var contentHandler: ((UNNotificationContent) -> Void)?
    var bestAttemptContent: UNMutableNotificationContent?

    override func didReceive(_ request: UNNotificationRequest, withContentHandler contentHandler: @escaping (UNNotificationContent) -> Void) {
        self.contentHandler = contentHandler
        bestAttemptContent = (request.content.mutableCopy() as? UNMutableNotificationContent)

        if let bestAttemptContent = bestAttemptContent {
            ALPushService.didReceiveNotificationExtensionRequest(request, with: bestAttemptContent, completionHandle: { (attach) in
                if let a = attach {
                    bestAttemptContent.userInfo = a
                    bestAttemptContent.body = a["ad_body"] as? String ?? ""

                    if let trackImp = (a["imps_tracking_url"] as? [String] ?? []).first, trackImp != "" {
                        print("tracImp URL: \(trackImp)")
                        self.sendDcm(with: trackImp)
                    }
                }
                contentHandler(bestAttemptContent)
            })
        }
    }

    override func serviceExtensionTimeWillExpire() {
        // Called just before the extension will be terminated by the system.
        // Use this as an opportunity to deliver your "best attempt" at modified content, otherwise the original push payload will be used.
        if let contentHandler = contentHandler, let bestAttemptContent = bestAttemptContent {
            contentHandler(bestAttemptContent)
        }
    }

    @objc func sendDcm(with url: String) {
        guard let u = URL(string: url) else { return }
        URLSession.shared.dataTask(with: URLRequest(url: u), completionHandler: { (data, response, error) in
            var canRetryNewImp: Bool = true
            if error != nil && canRetryNewImp {
                canRetryNewImp = false
                self.perform(#selector(self.sendDcm), with: url, afterDelay: 60)
            }
        }).resume()
    }

}
